@@ -22,12 +22,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
 #include <glib.h>
 
+#include "gibber-sockets.h"
 #include "gibber-xmpp-connection.h"
 #include "gibber-linklocal-transport.h"
 #include "gibber-util.h"
