@@ -33,7 +33,7 @@
 #define DEBUG_FLAG DEBUG_BYTESTREAM
 #include "gibber-debug.h"
 
-#include "signals-marshal.h"
+#include "gibber-signals-marshal.h"
 
 static void bytestream_iface_init (gpointer g_iface, gpointer iface_data);
 
@@ -42,16 +42,6 @@ G_DEFINE_TYPE_WITH_CODE (GibberBytestreamMuc, gibber_bytestream_muc,
     G_IMPLEMENT_INTERFACE (GIBBER_TYPE_BYTESTREAM_IFACE,
       bytestream_iface_init));
 
-/* signals */
-enum
-{
-  DATA_RECEIVED,
-  STATE_CHANGED,
-  LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = {0};
-
 /* properties */
 enum
 {
@@ -115,7 +105,7 @@ muc_connection_received_data_cb (GibberMucConnection *muc_connection,
     return;
 
   str = g_string_new_len ((const gchar *) data, length);
-  g_signal_emit (G_OBJECT (self), signals[DATA_RECEIVED], 0, sender,
+  g_signal_emit_by_name (G_OBJECT (self), "data-received", sender,
     str);
 
   g_string_free (str, TRUE);
@@ -254,7 +244,7 @@ gibber_bytestream_muc_set_property (GObject *object,
         if (priv->state != g_value_get_uint (value))
             {
               priv->state = g_value_get_uint (value);
-              g_signal_emit (object, signals[STATE_CHANGED], 0, priv->state);
+              g_signal_emit_by_name (object, "state-changed", priv->state);
             }
         break;
       default:
@@ -317,31 +307,9 @@ gibber_bytestream_muc_class_init (
       "Gibber MUC connection object used for communication by this "
       "bytestream if it's a muc one",
       GIBBER_TYPE_MUC_CONNECTION,
-      G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_MUC_CONNECTION,
       param_spec);
-
-  signals[DATA_RECEIVED] =
-    g_signal_new ("data-received",
-                  G_OBJECT_CLASS_TYPE (gibber_bytestream_muc_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  _gibber_signals_marshal_VOID__STRING_POINTER,
-                  G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
-
-  signals[STATE_CHANGED] =
-    g_signal_new ("state-changed",
-                  G_OBJECT_CLASS_TYPE (gibber_bytestream_muc_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__UINT,
-                  G_TYPE_NONE, 1, G_TYPE_UINT);
 }
 
 /*
@@ -418,7 +386,7 @@ gibber_bytestream_muc_close (GibberBytestreamIface *bytestream,
  *
  * Implements gibber_bytestream_iface_initiate on GibberBytestreamIface
  */
-gboolean
+static gboolean
 gibber_bytestream_muc_initiate (GibberBytestreamIface *bytestream)
 {
   /* Nothing to do */
