@@ -5,6 +5,8 @@
 
 #include "gibber-debug.h"
 
+#ifdef ENABLE_DEBUG
+
 static DebugFlags flags = 0;
 static gboolean initialized = FALSE;
 
@@ -20,6 +22,7 @@ static GDebugKey keys[] = {
   { "rmulticast-sender", DEBUG_RMULTICAST_SENDER },
   { "muc-connection",    DEBUG_MUC_CONNECTION    },
   { "bytestream",        DEBUG_BYTESTREAM        },
+  { "ft",                DEBUG_FILE_TRANSFER     },
   { "all",               ~0                      },
   { 0, },
 };
@@ -90,3 +93,5 @@ gibber_debug_stanza (DebugFlags flag,
       g_free (node_str);
     }
 }
+
+#endif
