@@ -35,8 +35,12 @@
 
 #include <gibber/gibber-xmpp-stanza.h>
 
+
 G_BEGIN_DECLS
 
+typedef struct _SalutPresenceCache SalutPresenceCache;
+typedef struct _SalutDisco SalutDisco;
+
 typedef struct _SalutConnection SalutConnection;
 typedef struct _SalutConnectionClass SalutConnectionClass;
 
@@ -52,6 +56,9 @@ struct _SalutConnection {
   TpPresenceMixin presence_mixin;
   TpContactsMixin contacts_mixin;
 
+  SalutPresenceCache *presence_cache;
+  SalutDisco *disco;
+
   /* Our name on the network */
   gchar *name;
 
