@@ -1002,28 +1002,36 @@ generate_tube_id (void)
 
 SalutTubeIface *
 salut_tubes_channel_tube_request (SalutTubesChannel *self,
-                                  const gchar *channel_type,
-                                  const gchar *service)
+    gpointer request_token,
+    GHashTable *request_properties,
+    gboolean require_new)
 {
   SalutTubesChannelPrivate *priv = SALUT_TUBES_CHANNEL_GET_PRIVATE (self);
   SalutTubeIface *tube;
+  const gchar *channel_type;
+  const gchar *service;
   guint tube_id;
   TpTubeType type;
   GHashTable *parameters;
 
   tube_id = generate_tube_id ();
 
-  if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  channel_type = tp_asv_get_string (request_properties,
+      TP_IFACE_CHANNEL ".ChannelType");
+
+  if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
       type = TP_TUBE_TYPE_STREAM;
+      service = tp_asv_get_string (request_properties,
+          TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+
     }
-/* Temporarily disabled since the implementation is incomplete */
-#if 0
-  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     {
       type = TP_TUBE_TYPE_DBUS;
+      service = tp_asv_get_string (request_properties,
+          TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
     }
-#endif
   else
     g_assert_not_reached ();
 
@@ -1108,7 +1116,7 @@ copy_tube_in_ptr_array (gpointer key,
   TpHandle initiator;
   gchar *service;
   GHashTable *parameters;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   TpTubeType type;
   GPtrArray *array = (GPtrArray *) user_data;
   GValue entry = {0,};
@@ -1186,6 +1194,7 @@ tube_closed_cb (SalutTubeIface *tube,
   SalutTubesChannel *self = SALUT_TUBES_CHANNEL (user_data);
   SalutTubesChannelPrivate *priv = SALUT_TUBES_CHANNEL_GET_PRIVATE (self);
   guint tube_id;
+  TpChannelManager *mgr;
 
   if (priv->closed)
     return;
@@ -1207,29 +1216,21 @@ tube_closed_cb (SalutTubeIface *tube,
   update_tubes_info (self);
 
   tp_svc_channel_type_tubes_emit_tube_closed (self, tube_id);
+  tp_svc_channel_emit_closed (tube);
 
-  /* FIXME: this is a workaround while new tube API is not implemented on
-   * D-Bus tubes */
-  if (SALUT_IS_TUBE_STREAM (tube))
+  if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
-      TpChannelManager *mgr;
-
-      tp_svc_channel_emit_closed (tube);
-
-      if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
-        {
-          g_object_get (priv->conn, "tubes-manager", &mgr, NULL);
-        }
-      else
-        {
-          g_object_get (priv->conn, "muc-manager", &mgr, NULL);
-        }
-
-      tp_channel_manager_emit_channel_closed_for_object (mgr,
-          TP_EXPORTABLE_CHANNEL (tube));
-
-      g_object_unref (mgr);
+      g_object_get (priv->conn, "tubes-manager", &mgr, NULL);
     }
+  else
+    {
+      g_object_get (priv->conn, "muc-manager", &mgr, NULL);
+    }
+
+  tp_channel_manager_emit_channel_closed_for_object (mgr,
+      TP_EXPORTABLE_CHANNEL (tube));
+
+  g_object_unref (mgr);
 }
 
 static void
@@ -1238,8 +1239,19 @@ tube_opened_cb (SalutTubeIface *tube,
 {
   SalutTubesChannel *self = SALUT_TUBES_CHANNEL (user_data);
   guint tube_id;
+  TpTubeType type;
 
-  g_object_get (tube, "id", &tube_id, NULL);
+  g_object_get (tube,
+      "id", &tube_id,
+      "type", &type,
+      NULL);
+
+  if (type == TP_TUBE_TYPE_DBUS)
+    {
+      add_yourself_in_dbus_names (self, tube_id);
+    }
+
+  update_tubes_info (self);
 
   tp_svc_channel_type_tubes_emit_tube_state_changed (self, tube_id,
       TP_TUBE_STATE_OPEN);
@@ -1294,7 +1306,7 @@ create_new_tube (SalutTubesChannel *self,
 {
   SalutTubesChannelPrivate *priv = SALUT_TUBES_CHANNEL_GET_PRIVATE (self);
   SalutTubeIface *tube;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   GibberMucConnection *muc_connection = NULL;
 
   if (self->muc != NULL)
@@ -1322,15 +1334,9 @@ create_new_tube (SalutTubesChannel *self,
 
   g_object_get (tube, "state", &state, NULL);
 
-  if (state == SALUT_TUBE_CHANNEL_STATE_OPEN)
-    {
-      /* FIXME: does it still make sense to call it here? */
-      update_tubes_info (self);
-    }
-
   /* The old API doesn't know the "not offered" state, so we have to wait that
    * the tube is offered before announcing it. */
-  if (state != SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED)
+  if (state != TP_TUBE_CHANNEL_STATE_NOT_OFFERED)
     {
       tp_svc_channel_type_tubes_emit_new_tube (self,
           tube_id,
@@ -1341,12 +1347,6 @@ create_new_tube (SalutTubesChannel *self,
           state);
     }
 
-  if (type == TP_TUBE_TYPE_DBUS &&
-      state != TP_TUBE_STATE_LOCAL_PENDING)
-    {
-      add_yourself_in_dbus_names (self, tube_id);
-    }
-
   g_signal_connect (tube, "tube-opened", G_CALLBACK (tube_opened_cb), self);
   g_signal_connect (tube, "tube-closed", G_CALLBACK (tube_closed_cb), self);
   g_signal_connect (tube, "tube-offered", G_CALLBACK (tube_offered_cb), self);
@@ -1540,7 +1540,7 @@ publish_tubes_in_node (gpointer key,
     (struct _i_hate_g_hash_table_foreach *) user_data;
   SalutTubesChannelPrivate *priv = SALUT_TUBES_CHANNEL_GET_PRIVATE (
       data->self);
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   GibberXmppNode *tube_node;
   TpTubeType type;
   TpHandle initiator;
@@ -1630,6 +1630,7 @@ salut_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
   TpBaseConnection *base;
   guint tube_id;
   SalutTubeIface *tube;
+  GError *err = NULL;
 
   g_assert (SALUT_IS_TUBES_CHANNEL (self));
 
@@ -1652,6 +1653,15 @@ salut_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
   tube = create_new_tube (self, TP_TUBE_TYPE_DBUS, priv->self_handle,
       TRUE, service, parameters, tube_id, 0, NULL);
 
+  if (!salut_tube_dbus_offer (SALUT_TUBE_DBUS (tube), &err))
+    {
+      salut_tube_iface_close (tube, TRUE);
+      dbus_g_method_return_error (context, err);
+
+      g_error_free (err);
+      return;
+    }
+
   tp_svc_channel_type_tubes_return_from_offer_d_bus_tube (context, tube_id);
 }
 
@@ -1669,7 +1679,7 @@ salut_tubes_channel_accept_d_bus_tube (TpSvcChannelTypeTubes *iface,
   SalutTubesChannel *self = SALUT_TUBES_CHANNEL (iface);
   SalutTubesChannelPrivate *priv;
   SalutTubeIface *tube;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   TpTubeType type;
   gchar *addr;
 
@@ -1713,8 +1723,6 @@ salut_tubes_channel_accept_d_bus_tube (TpSvcChannelTypeTubes *iface,
 
   salut_tube_iface_accept (tube, NULL);
 
-  update_tubes_info (self);
-
   g_object_get (tube,
       "dbus-address", &addr,
       NULL);
@@ -1774,7 +1782,7 @@ salut_tubes_channel_get_d_bus_tube_address (TpSvcChannelTypeTubes *iface,
   SalutTubeIface *tube;
   gchar *addr;
   TpTubeType type;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
 
   g_assert (SALUT_IS_TUBES_CHANNEL (self));
 
@@ -1854,7 +1862,7 @@ salut_tubes_channel_get_d_bus_names (TpSvcChannelTypeTubes *iface,
   GHashTable *names;
   GPtrArray *ret;
   TpTubeType type;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   guint i;
 
   g_assert (SALUT_IS_TUBES_CHANNEL (self));
@@ -1962,7 +1970,7 @@ send_channel_iq_tube (gpointer key,
   TpHandle initiator;
   gchar *service;
   GHashTable *parameters;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   TpTubeType type;
 
   g_object_get (tube,
@@ -1973,7 +1981,7 @@ send_channel_iq_tube (gpointer key,
                 "state", &state,
                 NULL);
 
-  if (state != SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED &&
+  if (state != TP_TUBE_CHANNEL_STATE_NOT_OFFERED &&
       salut_tube_iface_offer_needed (tube))
     {
       GError *error = NULL;
@@ -2175,7 +2183,7 @@ salut_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
   SalutTubesChannel *self = SALUT_TUBES_CHANNEL (iface);
   SalutTubesChannelPrivate *priv;
   SalutTubeIface *tube;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   TpTubeType type;
   GValue *address;
   GError *error = NULL;
@@ -2256,8 +2264,6 @@ salut_tubes_channel_accept_stream_tube (TpSvcChannelTypeTubes *iface,
       return;
     }
 
-  update_tubes_info (self);
-
   g_object_get (tube, "address", &address, NULL);
 
   tp_svc_channel_type_tubes_return_from_accept_stream_tube (context,
@@ -2279,7 +2285,7 @@ salut_tubes_channel_get_stream_tube_socket_address (TpSvcChannelTypeTubes *iface
   SalutTubesChannelPrivate *priv  = SALUT_TUBES_CHANNEL_GET_PRIVATE (self);
   SalutTubeIface *tube;
   TpTubeType type;
-  SalutTubeChannelState state;
+  TpTubeChannelState state;
   GValue *address;
   TpSocketAddressType address_type;
 
