@@ -1307,11 +1307,11 @@ create_new_outgoing_connection (SalutXmppConnectionManager *self,
 
   if (addrs->len == 0)
     {
-      DEBUG ("can't find addresse for contact %s", contact->name);
+      DEBUG ("can't find address for contact %s", contact->name);
 
       g_set_error (error, SALUT_XMPP_CONNECTION_MANAGER_ERROR,
           SALUT_XMPP_CONNECTION_MANAGER_ERROR_NO_ADDRESSE,
-          "can't find addresse for contact %s", contact->name);
+          "can't find address for contact %s", contact->name);
 
       g_array_free (addrs, TRUE);
       g_object_unref (connection);
